@@ -95,11 +95,9 @@ macro_rules! dyn2array {
 /// }
 ///
 /// let ljf: LJF = array_bytes::dyn_into!(*b"Love Jane Forever Forever Forever", 17);
-///
 /// assert_eq!(ljf, LJF(*b"Love Jane Forever"));
 ///
 /// let ljf: LJF = array_bytes::dyn_into!(b"Love Jane Forever Forever Forever".to_vec(), 17);
-///
 /// assert_eq!(ljf, LJF(*b"Love Jane Forever"));
 /// ```
 #[macro_export]
@@ -205,24 +203,6 @@ where
 	hex2bytes_unchecked(hex).try_into().unwrap()
 }
 
-/// Just like [`hex2array`] but without checking
-///
-/// # Examples
-///
-/// ```
-/// assert_eq!(
-/// 	array_bytes::hex2array_unchecked!("0x4c6f7665204a616e6520466f7265766572", 17),
-/// 	*b"Love Jane Forever"
-/// );
-/// ```
-#[deprecated(since = "1.2.0", note = "use `fn hex2array_unchecked` instead.")]
-#[macro_export]
-macro_rules! hex2array_unchecked {
-	($hex:expr, $len:expr) => {{
-		$crate::dyn2array!($crate::hex2bytes_unchecked($hex), $len)
-	}};
-}
-
 /// Try to convert [`Hex`] to a type directly
 ///
 /// # Examples
@@ -275,6 +255,34 @@ where
 	hex2array_unchecked(hex).into()
 }
 
+/// ```
+/// use serde::Deserialize;
+///
+/// #[derive(Debug, PartialEq)]
+/// struct LJF([u8; 17]);
+/// impl From<[u8; 17]> for LJF {
+/// 	fn from(array: [u8; 17]) -> Self {
+/// 		Self(array)
+/// 	}
+/// }
+///
+/// #[derive(Debug, PartialEq, Deserialize)]
+/// struct WrappedLJF {
+/// 	#[serde(deserialize_with = "array_bytes::hex_deserialize_into")]
+/// 	ljf: LJF,
+/// }
+///
+/// assert_eq!(
+/// 	serde_json::from_str::<WrappedLJF>(
+/// 		r#"{
+/// 		"ljf": "0x4c6f7665204a616e6520466f7265766572"
+/// 	}"#
+/// 	)
+/// 	.unwrap(),
+/// 	WrappedLJF {
+/// 		ljf: LJF(*b"Love Jane Forever")
+/// 	}
+/// );
 #[cfg(feature = "serde")]
 pub fn hex_deserialize_into<'de, D, T, const N: usize>(hex: D) -> Result<T, D::Error>
 where
@@ -284,8 +292,45 @@ where
 	Ok(hex2array_unchecked(<&str>::deserialize(hex)?).into())
 }
 
+/// Deserialize [`Hex`] to any Rust primitive num type
+///
+/// # Examples
+///
+/// ```
+/// use serde::Deserialize;
+///
+/// #[derive(Debug, PartialEq, Deserialize)]
+/// struct LJF {
+/// 	#[serde(deserialize_with = "array_bytes::de_hex2num")]
+/// 	_0: u8,
+/// 	#[serde(deserialize_with = "array_bytes::de_hex2num")]
+/// 	_1: u8,
+/// 	#[serde(deserialize_with = "array_bytes::de_hex2num")]
+/// 	_2: u8,
+/// 	#[serde(deserialize_with = "array_bytes::de_hex2num")]
+/// 	_3: u32,
+/// }
+///
+/// assert_eq!(
+/// 	serde_json::from_str::<LJF>(
+/// 		r#"{
+/// 		"_0": "0x5",
+/// 		"_1": "0x2",
+/// 		"_2": "0x0",
+/// 		"_3": "0x522"
+/// 	}"#
+/// 	)
+/// 	.unwrap(),
+/// 	LJF {
+/// 		_0: 5,
+/// 		_1: 2,
+/// 		_2: 0,
+/// 		_3: 1314
+/// 	}
+/// );
+/// ```
 #[cfg(feature = "serde")]
-pub fn hexd2num<'de, D, T>(hex: D) -> Result<T, D::Error>
+pub fn de_hex2num<'de, D, T>(hex: D) -> Result<T, D::Error>
 where
 	D: Deserializer<'de>,
 	T: TryFromHex,
@@ -295,8 +340,34 @@ where
 	T::try_from_hex(hex).map_err(|_| D::Error::custom(alloc::format!("Invalid hex str `{}`", hex)))
 }
 
+/// Deserialize [`Hex`] to [`Bytes`]
+///
+/// # Examples
+///
+/// ```
+/// use array_bytes::Bytes;
+/// use serde::Deserialize;
+///
+/// #[derive(Debug, PartialEq, Deserialize)]
+/// struct LJF {
+/// 	#[serde(deserialize_with = "array_bytes::de_hex2bytes")]
+/// 	ljf: Bytes,
+/// }
+///
+/// assert_eq!(
+/// 	serde_json::from_str::<LJF>(
+/// 		r#"{
+/// 		"ljf": "0x4c6f7665204a616e6520466f7265766572"
+/// 	}"#
+/// 	)
+/// 	.unwrap(),
+/// 	LJF {
+/// 		ljf: (*b"Love Jane Forever").to_vec()
+/// 	}
+/// );
+/// ```
 #[cfg(feature = "serde")]
-pub fn hexd2bytes<'de, D>(hex: D) -> Result<Bytes, D::Error>
+pub fn de_hex2bytes<'de, D>(hex: D) -> Result<Bytes, D::Error>
 where
 	D: Deserializer<'de>,
 {
@@ -379,11 +450,11 @@ mod test {
 	#[test]
 	fn hex2bytes_should_work() {
 		assert_eq!(
-			hex2bytes("4c6f7665204a616e6520466f7265766572").unwrap(),
+			hex2bytes("0x4c6f7665204a616e6520466f7265766572").unwrap(),
 			*b"Love Jane Forever"
 		);
 		assert_eq!(
-			hex2bytes("0x4c6f7665204a616e6520466f7265766572").unwrap(),
+			hex2bytes("4c6f7665204a616e6520466f7265766572").unwrap(),
 			*b"Love Jane Forever"
 		);
 
@@ -409,11 +480,11 @@ mod test {
 	#[test]
 	fn hex2bytes_unchecked_should_work() {
 		assert_eq!(
-			hex2bytes_unchecked("4c6f7665204a616e6520466f7265766572"),
+			hex2bytes_unchecked("0x4c6f7665204a616e6520466f7265766572"),
 			*b"Love Jane Forever"
 		);
 		assert_eq!(
-			hex2bytes_unchecked("0x4c6f7665204a616e6520466f7265766572"),
+			hex2bytes_unchecked("4c6f7665204a616e6520466f7265766572"),
 			*b"Love Jane Forever"
 		);
 	}
@@ -421,32 +492,23 @@ mod test {
 	#[test]
 	fn hex2array_should_work() {
 		assert_eq!(
-			hex2array("4c6f7665204a616e6520466f7265766572").unwrap(),
+			hex2array("0x4c6f7665204a616e6520466f7265766572").unwrap(),
 			*b"Love Jane Forever"
 		);
 		assert_eq!(
-			hex2array("0x4c6f7665204a616e6520466f7265766572").unwrap(),
+			hex2array("4c6f7665204a616e6520466f7265766572").unwrap(),
 			*b"Love Jane Forever"
 		);
 	}
 
 	#[test]
 	fn hex2array_unchecked_should_work() {
-		assert_eq!(
-			hex2array_unchecked("4c6f7665204a616e6520466f7265766572"),
-			*b"Love Jane Forever"
-		);
 		assert_eq!(
 			hex2array_unchecked("0x4c6f7665204a616e6520466f7265766572"),
 			*b"Love Jane Forever"
 		);
-
 		assert_eq!(
-			hex2array_unchecked!("4c6f7665204a616e6520466f7265766572", 17),
-			*b"Love Jane Forever"
-		);
-		assert_eq!(
-			hex2array_unchecked!("0x4c6f7665204a616e6520466f7265766572", 17),
+			hex2array_unchecked("4c6f7665204a616e6520466f7265766572"),
 			*b"Love Jane Forever"
 		);
 	}
@@ -457,6 +519,10 @@ mod test {
 			hex_try_into::<_, LJF, 17>("0x4c6f7665204a616e6520466f7265766572").unwrap(),
 			LJF(*b"Love Jane Forever")
 		);
+		assert_eq!(
+			hex_try_into::<_, LJF, 17>("4c6f7665204a616e6520466f7265766572").unwrap(),
+			LJF(*b"Love Jane Forever")
+		);
 	}
 
 	#[test]
@@ -465,17 +531,155 @@ mod test {
 			hex_into_unchecked::<_, LJF, 17>("0x4c6f7665204a616e6520466f7265766572"),
 			LJF(*b"Love Jane Forever")
 		);
+		assert_eq!(
+			hex_into_unchecked::<_, LJF, 17>("4c6f7665204a616e6520466f7265766572"),
+			LJF(*b"Love Jane Forever")
+		);
+	}
+
+	#[cfg(feature = "serde")]
+	#[test]
+	fn hex_deserialize_into_should_work() {
+		#[derive(Debug, PartialEq, Deserialize)]
+		struct WrappedLJF {
+			#[serde(deserialize_with = "hex_deserialize_into")]
+			ljf: LJF,
+		}
+
+		assert_eq!(
+			serde_json::from_str::<WrappedLJF>(
+				r#"{
+				"ljf": "0x4c6f7665204a616e6520466f7265766572"
+			}"#
+			)
+			.unwrap(),
+			WrappedLJF {
+				ljf: LJF(*b"Love Jane Forever")
+			}
+		);
+		assert_eq!(
+			serde_json::from_str::<WrappedLJF>(
+				r#"{
+				"ljf": "4c6f7665204a616e6520466f7265766572"
+			}"#
+			)
+			.unwrap(),
+			WrappedLJF {
+				ljf: LJF(*b"Love Jane Forever")
+			}
+		);
+	}
+
+	#[cfg(feature = "serde")]
+	#[test]
+	fn de_hex2num_should_work() {
+		macro_rules! assert_de_hex2num {
+			($num_type:ty) => {{
+				#[derive(Debug, PartialEq, Deserialize)]
+				struct LJF {
+					#[serde(deserialize_with = "de_hex2num")]
+					_0: $num_type,
+					#[serde(deserialize_with = "de_hex2num")]
+					_1: $num_type,
+					#[serde(deserialize_with = "de_hex2num")]
+					_2: $num_type,
+					#[serde(deserialize_with = "de_hex2num")]
+					_3: u32,
+				}
+
+				assert_eq!(
+					serde_json::from_str::<LJF>(
+						r#"{
+						"_0": "0x5",
+						"_1": "0x2",
+						"_2": "0x0",
+						"_3": "0x522"
+					}"#
+					)
+					.unwrap(),
+					LJF {
+						_0: 5,
+						_1: 2,
+						_2: 0,
+						_3: 1314
+					}
+				);
+				assert_eq!(
+					serde_json::from_str::<LJF>(
+						r#"{
+						"_0": "5",
+						"_1": "2",
+						"_2": "0",
+						"_3": "522"
+					}"#
+					)
+					.unwrap(),
+					LJF {
+						_0: 5,
+						_1: 2,
+						_2: 0,
+						_3: 1314
+					}
+				);
+			}};
+		}
+
+		assert_de_hex2num!(isize);
+		assert_de_hex2num!(i8);
+		assert_de_hex2num!(i16);
+		assert_de_hex2num!(i32);
+		assert_de_hex2num!(i64);
+		assert_de_hex2num!(i128);
+		assert_de_hex2num!(usize);
+		assert_de_hex2num!(u8);
+		assert_de_hex2num!(u16);
+		assert_de_hex2num!(u32);
+		assert_de_hex2num!(u64);
+		assert_de_hex2num!(u128);
+	}
+
+	#[cfg(feature = "serde")]
+	#[test]
+	fn de_hex2bytes_should_work() {
+		#[derive(Debug, PartialEq, Deserialize)]
+		struct LJF {
+			#[serde(deserialize_with = "de_hex2bytes")]
+			ljf: Bytes,
+		}
+
+		assert_eq!(
+			serde_json::from_str::<LJF>(
+				r#"{
+				"ljf": "0x4c6f7665204a616e6520466f7265766572"
+			}"#
+			)
+			.unwrap(),
+			LJF {
+				ljf: (*b"Love Jane Forever").to_vec()
+			}
+		);
+		assert_eq!(
+			serde_json::from_str::<LJF>(
+				r#"{
+				"ljf": "4c6f7665204a616e6520466f7265766572"
+			}"#
+			)
+			.unwrap(),
+			LJF {
+				ljf: (*b"Love Jane Forever").to_vec()
+			}
+		);
 	}
 
 	#[test]
 	fn bytes2hex_should_work() {
-		assert_eq!(
-			bytes2hex("", b"Love Jane Forever"),
-			Hex::from("4c6f7665204a616e6520466f7265766572")
-		);
 		assert_eq!(
 			bytes2hex("0x", b"Love Jane Forever"),
 			Hex::from("0x4c6f7665204a616e6520466f7265766572")
 		);
+		assert_eq!(
+			bytes2hex("", b"Love Jane Forever"),
+			Hex::from("4c6f7665204a616e6520466f7265766572")
+		);
 	}
 }
